//+------------------------------------------------------------------+
//|                                                Dolly EA v1.0.mq4 |
//|                                        Copyright  2007, Gotenkz |
//+------------------------------------------------------------------+
#define MAGICEA  17012007

extern string DollyEA1 = "-----Primary Setting-----";
extern bool DailyOpenCalculate = false;
extern int TakeProfit          = 20;
extern int StopLoss            = 40;
extern int TrailingStop        = 15;

extern string DollyEA2 = "-----Money Management Setting-----";
extern double Lots           = 0.1;
extern double MaximumRisk    = 0.02;
extern double DecreaseFactor = 3;

extern string DollyEA3 = "-----Trade Hour Setting-----";
extern bool UseTradeHour  = false;
extern int  FromTradeHour = 8;
extern int  ToTradeHour   = 18;
//+------------------------------------------------------------------+
//| calculate open positions                                         |
//+------------------------------------------------------------------+
int CalculateCurrentOrders(string symbol)
  {
//----
   int buys=0,sells=0;

   for(int i=0;i<OrdersTotal();i++)
   {
    if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==false) break;
    if(OrderSymbol()==Symbol() && OrderMagicNumber()==MAGICEA)
    {
     if(OrderType()==OP_BUY)  buys++;
     if(OrderType()==OP_SELL) sells++;
    }
   }
   //---- return orders volume
   if(buys>0) return(buys);
   else       return(-sells);
//----
  }
//+------------------------------------------------------------------+
//| calculate optimal lot size                                       |
//+------------------------------------------------------------------+
double LotsOptimized()
  {
//----
   double lot=Lots;
   int    orders=HistoryTotal(); // history orders total
   int    losses=0;              // number of losses orders without a break
   
   lot=NormalizeDouble(AccountFreeMargin()*MaximumRisk/1000.0,1);
   
   //---- calcuulate number of losses orders without a break
   if(DecreaseFactor>0)
   {
    for(int i=orders-1;i>=0;i--)
    {
     if(OrderSelect(i,SELECT_BY_POS,MODE_HISTORY)==false) { Print("Error in history!"); break; }
     if(OrderSymbol()!=Symbol() || OrderType()>OP_SELL) continue;
     //----
     if(OrderProfit()>0) break;
     if(OrderProfit()<0) losses++;
    }
    if(losses>1) lot=NormalizeDouble(lot-lot*losses/DecreaseFactor,1);
   }     
   //---- return lot size
   if(lot<0.1) lot=0.1;
   return(lot);
//----
  }
//+------------------------------------------------------------------+
//| expert deinitialization function                                 |
//+------------------------------------------------------------------+
int deinit()
  {
//----
   ObjectsDeleteAll(0,OBJ_RECTANGLE);
   ObjectsDeleteAll(0,OBJ_TEXT);
   ObjectDelete("BUY");
   ObjectDelete("SELL");
   ObjectDelete("BUYv");
   ObjectDelete("SELLv");
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| expert start function                                            |
//+------------------------------------------------------------------+
int start()
  {
//----
   //---- time for trading
   if (UseTradeHour)
   {
    if(!(Hour()>=FromTradeHour&&Hour()<=ToTradeHour))
    {
     return(0);
    }
   }

   double Stoch_M,Stoch_S,MACD_M,MACD_S,RSI,P,open,B1,B2,S1,S2;
   int    act;

   Stoch_M = iStochastic(NULL,PERIOD_H4,8,3,3,1,0,MODE_MAIN,0);
   Stoch_S = iStochastic(NULL,PERIOD_H4,8,3,3,1,0,MODE_SIGNAL,0);
   MACD_M  = iMACD(NULL,PERIOD_H4,8,17,9,PRICE_CLOSE,MODE_MAIN,0);
   MACD_S  = iMACD(NULL,PERIOD_H4,8,17,9,PRICE_CLOSE,MODE_SIGNAL,0);
   RSI     = iRSI(NULL,PERIOD_H4,10,PRICE_CLOSE,0);
   
   P    = (iHigh(NULL,PERIOD_D1,1) + iLow(NULL,PERIOD_D1,1) + iClose(NULL,PERIOD_D1,1))/3;
   open = iOpen(NULL,PERIOD_D1,0);
   
   if(DailyOpenCalculate==true){P=open;}
   {
    B1 = P+20*Point;
    B2 = P+40*Point;
    S1 = P-20*Point;
    S2 = P-40*Point;
   }
   
   //if(Bars<100 || IsTradeAllowed()==false) return;
   
   if(CalculateCurrentOrders(Symbol())==0)
   {
    //CHECK FOR OPEN
    if(OrdersTotal()<1)
    {
     //---- buy conditions
     if(Ask>=B1&&Ask<=B2&&Stoch_M>Stoch_S&&Stoch_S>20&&Stoch_S<80&&MACD_M>MACD_S&&RSI>50) //open buy between B1 & B2
     {
      act=OrderSend(Symbol(),OP_BUY,LotsOptimized(),Ask,3,Ask-StopLoss*Point,Ask+TakeProfit*Point,"",MAGICEA,0,Blue);
      return;
     }
     //---- sell conditions
     if(Bid<=S1&&Bid>=S2&&Stoch_M<Stoch_S&&Stoch_S>20&&Stoch_S<80&&MACD_M<MACD_S&&RSI<50) //open sell between S1 & S2
     {
      act=OrderSend(Symbol(),OP_SELL,LotsOptimized(),Bid,3,Bid+StopLoss*Point,Bid-TakeProfit*Point,"",MAGICEA,0,Red);
      return;
     }
    }
   }
   else
   {
    //CHECK FOR CLOSE
    for(int i=0;i<OrdersTotal();i++)
    {
     if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==false)        break;
     if(OrderMagicNumber()!=MAGICEA || OrderSymbol()!=Symbol()) continue;
      
     //---- check order type 
     if (OrderSymbol()==Symbol() && OrderType()==OP_BUY && Bid-TrailingStop*Point > OrderStopLoss())
     {
      if (Stoch_M>Stoch_S&&Stoch_S>20&&Stoch_S<80&&MACD_M>MACD_S&&RSI>50)
      {
       if (Bid > OrderOpenPrice()+TrailingStop*Point)
       {
        OrderModify(OrderTicket(),OrderOpenPrice(),Bid-TrailingStop*Point,0,0,White);
       }
      }        
     }      
     if (OrderSymbol()==Symbol() && OrderType()==OP_SELL && Ask-TrailingStop*Point < OrderStopLoss())
     {   
      if (Stoch_M<Stoch_S&&Stoch_S>20&&Stoch_S<80&&MACD_M<MACD_S&&RSI<50)
      {
       if (Ask < OrderOpenPrice()-TrailingStop*Point)
       {
        OrderModify(OrderTicket(),OrderOpenPrice(),Ask+TrailingStop*Point,0,0,White);
       }
      }         
     }
    }
   }
   
   //---- graphics for buy/sell area
   Graphics("BUY1",B1,B2,Blue);
   Graphics("BUY2",B2,B2,Blue);
   Graphics("SELL1",S1,S2,Red);
   Graphics("SELL2",S2,S2,Red);
   
   //---- show price value in graphics
   if(ObjectFind("B1") != 0)
   {
    ObjectCreate("B1",OBJ_TEXT,0,Time[0],B1);
    ObjectSetText("B1", "              "+DoubleToStr(B1,Digits)+"", 8, "Arial Bold",White);
   }
   else
   {
    ObjectMove("B1",0,Time[0],B1);
   }   
   if(ObjectFind("B2") != 0)
   {
    ObjectCreate("B2",OBJ_TEXT,0,Time[0],B2);
    ObjectSetText("B2", "              "+DoubleToStr(B2,Digits)+"", 8, "Arial Bold",White);
   }
   else
   {
    ObjectMove("B2",0,Time[0],B2);
   }   
   if(ObjectFind("S1") != 0)
   {
    ObjectCreate("S1",OBJ_TEXT,0,Time[0],S1);
    ObjectSetText("S1", "              "+DoubleToStr(S1,Digits)+"", 8, "Arial Bold",White);
   }
   else
   {
    ObjectMove("S1",0,Time[0],S1);
   }   
   if(ObjectFind("S2") != 0)
   {
    ObjectCreate("S2",OBJ_TEXT,0,Time[0],S2);
    ObjectSetText("S2", "              "+DoubleToStr(S2,Digits)+"", 8, "Arial Bold",White);
   }
   else
   {
    ObjectMove("S2",0,Time[0],S2);
   }
   
   //----
   ObjectCreate("BUY",OBJ_LABEL,0,0,0);
   ObjectSetText("BUY","BUY AREA >>",8,"Arial Bold",Blue);
   ObjectSet("BUY",OBJPROP_CORNER,0);
   ObjectSet("BUY",OBJPROP_XDISTANCE,7);
   ObjectSet("BUY",OBJPROP_YDISTANCE,24); 
   
   ObjectCreate("SELL",OBJ_LABEL,0,0,0);
   ObjectSetText("SELL","SELL AREA >>",8,"Arial Bold",Red);
   ObjectSet("SELL",OBJPROP_CORNER,0);
   ObjectSet("SELL",OBJPROP_XDISTANCE,7);
   ObjectSet("SELL",OBJPROP_YDISTANCE,36); 
   
   ObjectCreate("BUYv",OBJ_LABEL,0,0,0);
   ObjectSetText("BUYv",""+DoubleToStr(B1,Digits)+" - "+DoubleToStr(B2,Digits)+"",8,"Arial Bold",White);
   ObjectSet("BUYv",OBJPROP_CORNER,0);
   ObjectSet("BUYv",OBJPROP_XDISTANCE,80);
   ObjectSet("BUYv",OBJPROP_YDISTANCE,24); 
   
   ObjectCreate("SELLv",OBJ_LABEL,0,0,0);
   ObjectSetText("SELLv",""+DoubleToStr(S1,Digits)+" - "+DoubleToStr(S2,Digits)+"",8,"Arial Bold",White);
   ObjectSet("SELLv",OBJPROP_CORNER,0);
   ObjectSet("SELLv",OBJPROP_XDISTANCE,80);
   ObjectSet("SELLv",OBJPROP_YDISTANCE,36); 
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| show graphics function                                           |
//+------------------------------------------------------------------+
void Graphics(string GFX, double start, double end, color col)
  {
//----
   ObjectCreate(GFX, OBJ_RECTANGLE, 0, iTime(NULL,PERIOD_D1,0), start, Time[0],end);
   ObjectSet(GFX, OBJPROP_COLOR, col);
   ObjectSet(GFX, OBJPROP_BACK, true);
   ObjectMove(GFX, 1, Time[0],end);
   ObjectMove(GFX, 0, iTime(NULL,PERIOD_D1,0), start);
//----
  }